@@ -36,13 +36,47 @@ std::string htmCircleRegion (size_t level, double ra, double dec, double radius,
    return s.str();
 }
 
-unsigned long htmID (size_t level, double ra, double dec)
+unsigned long long htmID (size_t level, double ra, double dec)
 {
    htmInterface htm(level, 5);
    uint64 id=htm.lookupID(ra, dec);
    return id;
 }
 
+std::string htmName (size_t level, double ra, double dec)
+{
+   htmInterface htm(level, 5);
+   uint64 id=htm.lookupID(ra, dec);
+   std::string name = htm.lookupName(id);
+   return name;
+}
+
+std::vector<unsigned long long> htmIDBulk (size_t level, std::vector< std::pair <double, double> > coords)
+{
+    std::vector<uint64> bulkIDs;
+    htmInterface htm(level, 5);
+    for(size_t idx=0; idx<coords.size(); idx++)
+    {
+        uint64 id=htm.lookupID(coords[idx].first, coords[idx].second);
+        bulkIDs.push_back(id);
+    }
+    return bulkIDs;
+}
+
+std::vector<std::string> htmNameBulk (size_t level, std::vector< std::pair <double, double> > coords)
+{
+    std::vector<std::string> bulkNames;
+    std::string name;
+    htmInterface htm(level, 5);
+    for(size_t idx=0; idx<coords.size(); idx++)
+    {
+        uint64 id=htm.lookupID(coords[idx].first, coords[idx].second);
+        name = htm.lookupName(id);
+        bulkNames.push_back(name);
+    }
+    return bulkNames;
+}
+
 int main(int argc, char *argv[]) {
    int args              =1;
    size_t level          = atoi(argv[args++]);
